#!/bin/bash
set -euxo pipefail

# Print which dts2repl commits are being compared
OLD_COMMIT="$(<ci-output/dts2repl.version)"
echo "Dashboard repls were generated by dts2repl $OLD_COMMIT, commits since then are:"
git log --oneline --graph --color=always "$OLD_COMMIT..HEAD"

cd ci-output
# Generate repls for the boards that have downloaded dtses for comparison
(
    cd repls/generated
    # process platforms that have a system clock override in a config.json file
    mkdir ../../dts-clock-override
    grep -Hr override-system-clock-frequency ../../repls/dashboard/*.json | cut -d/ -f5 | cut -d- -f1 | parallel 'mv ../../dts/{}.dts ../../dts-clock-override/'
    ls -1 ../../dts-clock-override | cut -d. -f1 | parallel 'dts2repl ../../dts-clock-override/{}.dts --override-system-clock-frequency=$(jq -r ".[\"override-system-clock-frequency\"]" ../dashboard/{}-config.json) --output {}.repl'
    # process other platforms
    ls -1 ../../dts | cut -d. -f1 | parallel 'dts2repl ../../dts/{}.dts --output {}.repl'
    # merge results
    mv ../../dts-clock-override/* ../../dts
    rmdir ../../dts-clock-override
)

# Save diffs for repls that are different
# diff returns 0 on no difference which we use to delete diff files if the
# generated repl is identical (+/- blank lines) to the dashboard one
pushd repls/diffs
ls -1 ../generated | parallel 'diff -u --new-file --ignore-blank-lines "../dashboard/{}" "../generated/{}" > "{}.diff" && rm "{}.diff" || true'
popd

if [ "$(ls -A repls/diffs | wc -l)" != 0 ]; then
    echo "Found differences, running Renode"
    RENODE_LOCATION=$(cat ${CI_PROJECT_DIR}/renode-location)
    echo "Renode should be at ${RENODE_LOCATION}"
    export PATH=$(find ${RENODE_LOCATION} -type f -name renode-test | xargs realpath | xargs dirname):$PATH
    renode-test --results-dir robot-results "$PWD/../ci/load_repls_with_diffs.robot"
fi
